
package com.tmerclub.cloud.platform.mapper;

import com.tmerclub.cloud.api.platform.vo.SysUserVO;
import com.tmerclub.cloud.platform.dto.SysUserDTO;
import com.tmerclub.cloud.platform.model.SysUser;
import com.tmerclub.cloud.platform.vo.SysUserSimpleVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @author lhd
 * @date 2020/12/22
 */
public interface SysUserMapper {

    /**
     * 根据用户id获取当前登陆的商家用户信息
     *
     * @param userId 用户id
     * @return 商家用户信息
     */
    SysUserSimpleVO getSimpleByUserId(@Param("userId") Long userId);

    /**
     * 获取平台用户列表
     *
     * @param sysUserDTO 搜索参数
     * @return 平台用户列表
     */
    List<SysUserVO> listByShopId(@Param("sysUser") SysUserDTO sysUserDTO);

    /**
     * 根据用户id获取商家用户信息
     *
     * @param userId 用户id
     * @return 商家用户信息
     */
    SysUserVO getByUserId(@Param("userId") Long userId);

    /**
     * 保存商家用户信息
     *
     * @param sysUser
     */
    void save(@Param("sysUser") SysUser sysUser);

    /**
     * 更新平台用户信息
     *
     * @param sysUser
     */
    void update(@Param("sysUser") SysUser sysUser);

    /**
     * 根据平台用户id删除平台用户
     *
     * @param sysUserId
     */
    void deleteById(@Param("sysUserId") Long sysUserId);

    /**
     * 根据昵称统计已有的数量
     *
     * @param nickName
     * @param sysUserId
     * @return
     */
    int countNickName(@Param("nickName") String nickName, @Param("sysUserId") Long sysUserId);

    /**
     * 根据用户id列表查名称
     *
     * @param userIds 用户id列表
     * @return
     */
    List<SysUserVO> getUserName(@Param("userIds") List<Long> userIds);

    /**
     * 更新电话号码
     *
     * @param sysUserId
     * @param phoneNum
     */
    void updatePhoneNum(@Param("sysUserId") Long sysUserId, @Param("phoneNum") String phoneNum);
}
