
package com.tmerclub.cloud.supplier.mapper;

import com.tmerclub.cloud.supplier.dto.AuditingSupplierInfoDTO;
import com.tmerclub.cloud.supplier.model.SupplierAuditing;
import com.tmerclub.cloud.supplier.vo.SupplierAuditingInfoVO;
import com.tmerclub.cloud.supplier.vo.SupplierAuditingVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 供应商审核信息
 *
 * @author lhd
 * @date 2022-03-14 17:04:32
 */
public interface SupplierAuditingMapper {

    /**
     * 获取商家审核信息列表
     *
     * @return 商家审核信息列表
     */
    List<SupplierAuditingVO> list();

    /**
     * 根据商家审核信息id获取商家审核信息
     *
     * @param supplierAuditingId 商家审核信息id
     * @return 商家审核信息
     */
    SupplierAuditingVO getBySupplierAuditingId(@Param("supplierAuditingId") Long supplierAuditingId);

    /**
     * 保存商家审核信息
     *
     * @param supplierAuditing 商家审核信息
     */
    void save(@Param("supplierAuditing") SupplierAuditing supplierAuditing);

    /**
     * 更新商家审核信息
     *
     * @param supplierAuditing 商家审核信息
     */
    void update(@Param("supplierAuditing") SupplierAuditing supplierAuditing);

    /**
     * 根据商家审核信息id删除商家审核信息
     *
     * @param supplierAuditingId
     */
    void deleteById(@Param("supplierAuditingId") Long supplierAuditingId);

    /**
     * 获取指定供应商的申请信息
     *
     * @param supplierId
     * @return
     */
    SupplierAuditingVO getBySupplierId(@Param("supplierId") Long supplierId);

    /**
     * 获取供应商申请列表
     * @param auditingInfoDTO
     * @return
     */
    List<SupplierAuditingInfoVO> auditingInfoList(@Param("auditingInfo") AuditingSupplierInfoDTO auditingInfoDTO);

    /**
     * 更新开店失败
     * @param supplierIds
     */
    void updateToFail(@Param("supplierIds") List<Long> supplierIds);
}
